#!/usr/bin/perl

require 'zfsmgr-lib.pl';
&ReadParse();

use Text::Trim qw(trim);

my $url = "index.cgi";
my $err = "";
my $sep = "<br>";


sub zfs_chown
{
	my ($user, $group, $ds) = @_;
	my (undef, undef, $uid, $gid) = getpwnam($user);
	my $owner = $user . ":";

	if (!defined $uid || !defined $gid) {
		$err = "Unknown user name or group";
		my %param = ('error' => $err);
		webmin_log("Change", "owner", "error: $err for $ds", \%param);
		$err = $text{'err_unknown_owner'};
		return (-1);
	}
	if ($group ne "") {
		$gid = getgrnam($group);
		if (!defined $gid) {
			$err = text("err_unknown_group", $group);
			my %param = ('error' => $err);
			webmin_log("Change", "owner", "error: $err for $ds", \%param);
			return (-1);
		}
		$owner .= $group;
	} else {
		$owner .= getgrgid($gid);
	}
	my @res = &backquote_command("zfs get -Hpo value mountpoint $ds");
	if ($? != 0) {
		# ошибка
		if ($? != -1) {
			$err = join($sep, @res);
			my %param = ('error' => $err);
			my $owner = $user;
			if ($group ne "") {
				$owner .= ":".$group;
			}
			webmin_log("Change", "owner", "error: $err for $ds", \%param);
		}
		$err = $text{'err_mountpoint', $ds};
	} else {
		# ошибок нет, всё прошло
		chomp @res;
		my $mnt = $res[0];
		my $restore = 0;
		@res = &backquote_command("zfs get -Hpo value readonly $ds");
		chomp @res;
		if ($res[0] eq "on") {
			$restore = 1;
			&backquote_command("zfs set readonly=off $ds");
		}
		chown $uid, $gid, $mnt;
		webmin_log("Change", "owner", "chown $owner $mnt");
		if ($restore) {
			&backquote_command("zfs set readonly=on $ds");
		}
	}

	return (0);
}


sub zfs_chmod
{
	my ($md, $ds) = @_;
	my $str = sprintf "%o", $md;

	my @res = &backquote_command("zfs get -Hpo value mountpoint $ds");
	chomp @res;
	my $mnt = $res[0];
	if ($? != 0) {
		# ошибка
		if ($? != -1) {
			$err = join($sep, @res);
			my %param = ('error' => $err);
			webmin_log("Change", "mode", "error: $err for $ds", \%param);
		}
		$err = $text{'err_mountpoint', $ds};
	} else {
		# ошибок нет, всё прошло
		my $restore = 0;
		@res = &backquote_command("zfs get -Hpo value readonly $ds");
		chomp @res;
		if ($res[0] eq "on") {
			$restore = 1;
			&backquote_command("zfs set readonly=off $ds");
		}
		my $cnt = chmod($md & 0777, $mnt);
		webmin_log("Change", "mode", "chmod $str $mnt");
		if ($restore) {
			&backquote_command("zfs set readonly=on $ds");
		}
	}

	return (0);
}



if ($in{'zfs'} eq "") {
	# error
} elsif ($in{'cmd'} eq "") {
	$err = $text{"err_empty_command"};
	my %param = ('error' => $err);
	webmin_log("Command", "undefined", trim($in{'zfs'}), \%param);
	$url .= "?cmd=list";
} elsif ($in{'cmd'} eq 'create') {
	$url .= "?";
	my $type, $cmd, $name;
	if ($in{"blocksize"} ne "") {
		$type = "volume";
		$cmd = "zvol_create";
		if (!has_access('uvol_create')) {
			goto NOPERM;
		}
	} else {
		$type = "dataset";
		$cmd = "zfs_create";
		if (!has_access('uzfs_create')) {
			goto NOPERM;
		}
	}
	if ($type eq "volume") {
		$name = trim($in{"volume"});
		if ($name eq "") {
			$url .= "cmd=$cmd&zfs=$in{'zfs'}";
			$err = $text{"err_empty_volume_name"};
			my %param = ('error' => $err);
			webmin_log("Create", "volume", "error: $err for ".trim($in{'zfs'}), \%param);
			goto OUT;
		} elsif ($in{"volsize"} eq "") {
			$url .= "cmd=$cmd&zfs=$in{'zfs'}";
			$err = $text{"err_empty_volsize"};
			my %param = ('error' => $err);
			webmin_log("Create", "volume", "error: $err for ".trim($in{'zfs'}), \%param);
			goto OUT;
		} elsif ($in{"blocksize"} eq "") {
			$url .= "cmd=$cmd&zfs=$in{'zfs'}";
			$err = $text{"err_empty_blocksize"};
			my %param = ('error' => $err);
			webmin_log("Create", "volume", "error: $err for ".trim($in{'zfs'}), \%param);
			goto OUT;
		}
	} elsif ($type eq "dataset") {
		$name = trim($in{"dataset"});
		if ($name eq "") {
			$url .= "cmd=$cmd&zfs=$in{'zfs'}";
			$err = $text{"err_empty_dataset_name"};
			my %param = ('error' => $err);
			webmin_log("Create", "dataset", "error: $err for ".trim($in{'zfs'}), \%param);
			goto OUT;
		}
	}

	if (!zfs_allowed(trim($in{'zfs'}))) {
		$url .= "cmd=prop&zfs=$in{'zfs'}";
		$err = text("err_forbidden", $in{'zfs'}),;
		my %param = ('error' => $err);
		webmin_log("Create", $type, "error: $err for ".trim($in{'zfs'}), \%param);
		goto OUT;
	}
	my @res;
	my $ds = trim($in{'zfs'})."/".$name;
	if (length($ds) > zfs_max_length()) {
		$url .= "cmd=prop&zfs=$in{'zfs'}";
		$err = text("err_toolong", $in{'zfs'}),;
		my %param = ('error' => $err);
		webmin_log("Create", $type, "error: $err for ".trim($in{'zfs'}), \%param);
		goto OUT;
	}

	if ($type eq "volume") {
		@res = &backquote_command("zfs get -Hpo value name $ds 2>&1");
		if ($? == 0) {
			$err = text("err_volume_exists", $name, trim($in{'zfs'}));
			$url .= "cmd=$cmd&zfs=$in{'zfs'}";
			my %param = ('error' => $err);
			webmin_log("Create", "volume", "error: $err for $ds", \%param);
			goto OUT;
		}
	} else {
		@res = &backquote_command("zfs get -Hpo value mountpoint $in{'zfs'}");
		chomp @res;
		my $dir = "$res[0]/".$name;
		if (-e $dir || -f $dir) {
			$err = text("err_dir_exists", $dir, $in{'zfs'});
			$url .= "cmd=$cmd&zfs=$in{'zfs'}";
			my %param = ('error' => $err);
			webmin_log("Create", "dataset", "error: $err for $ds", \%param);
			goto OUT;
		}
		@res = &backquote_command("zfs get -Hpo value name $ds 2>&1");
		if ($? == 0) {
			$err = text("err_dataset_exists", $name, trim($in{'zfs'}));
			$url .= "cmd=$cmd&zfs=$in{'zfs'}";
			my %param = ('error' => $err);
			webmin_log("Create", "dataset", "error: $err for $ds", \%param);
			goto OUT;
		}
	}
	my $zcmd = "zfs create -p";
	if ($type eq "volume") {
		$val = trim($in{'blocksize'});
		$zcmd .= " -b'$val'";
		$val = trim($in{'volsize'});
		$zcmd .= " -V'$val' -o refreservation='$val'";
	} else {
		my $val = trim($in{'quota'});
		if (($val ne "") && ($val ne "inherit")) {
			if ($val <= 0) {
				$val = "none";
			}
			$zcmd .= " -o quota='$val'";
		}
		$val = trim($in{'reservation'});
		if (($val ne "") && ($val ne "inherit")) {
			if ($val <= 0) {
				$val = "none";
			}
			$zcmd .= " -o reservation='$val'";
		}
		$val = trim($in{'recordsize'});
		if (($val ne "") && ($val ne "inherit")) {
			$zcmd .= " -o recordsize='$val'";
		}
		$val = trim($in{'sharenfs'});
		if ($val ne "") {
			$zcmd .= " -o sharenfs='$val'";
		}
		$zcmd .= " -o devices=off";
		$zcmd .= " -o exec=off";
		$zcmd .= " -o setuid=off";
	}
	$val = trim($in{'compression'});
	if (($val ne "") && ($val ne "inherit"))  {
		$zcmd .= " -o compression='$val'";
	}
	$val = trim($in{'readonly'});
	if (($val ne "") && ($val ne "inherit"))  {
		$zcmd .= " -o readonly='$val'";
	}
	$zcmd .= " -o dedup=off";
	$zcmd .= " '$ds'";
	@res = &backquote_command($zcmd." 2>&1");
	if ($? != 0) {
		# error
		$err = join($sep, @res);
		$url .= "cmd=$cmd&zfs=$in{'zfs'}";
		my %param = ('error' => $err);
		webmin_log("Create", $type, "error: $err for $ds", \%param);
		chomp @res;
		if ($res[0] =~ /out of space/) {
			$err = text("err_create_space", $ds);
		} else {
			$err = text("err_create", $ds);
		}
	} else {
		# no errors, Ok
		$url .= "cmd=prop&zfs=$ds";
		webmin_log("Create", $type, $zcmd);
		if ($type eq "dataset") {
			# chown
			my $user = $in{'user'};
			my $group = $in{'group'};
			my $owner = $user.":";
			if ($user ne "") {
				zfs_chown($user, $group, $ds);
			}
			# chmod
			my $mode = oct($in{'mode'});
			if (defined $mode && ($mode > 0) && ($mode <= 0777)) {
				zfs_chmod($mode, $ds);
			}
		}
	}
} elsif ($in{'cmd'} eq 'destroy') {
	my $ds = trim($in{'zfs'});
	my @res = &backquote_command("zfs get -Hpo value type '$ds' 2>&1");
	if ($? != 0) {
		$err = join($sep, @res);
		$url .= "?cmd=zfs_destroy&zfs=$ds";
		my %param = ('error' => $err);
		webmin_log("Get", "type", "error: $err for $ds", \%param);
		$err = text('err_get_type', $ds);
		goto OUT;
	}
	chomp @res;
	my $type = $res[0], $link, $obj;
	if (!zfs_allowed($ds)) {
		$url .= "cmd=$cmd&zfs=$ds";
		$err = text("err_forbidden", $ds),;
		my %param = ('error' => $err);
		webmin_log("Destroy", $type, "error: $err for ".$ds, \%param);
		goto OUT;
	}
	if ($type eq "volume") {
		if (!has_access('uvol_destroy')) {
			goto NOPERM;
		}
		$link = "zvol_destroy";
		$obj = "zvol";
	} elsif ($type eq "filesystem") {
		if (!has_access('uzfs_destroy')) {
			goto NOPERM;
		}
		$link = "zfs_destroy";
		$obj = "dataset";
	} else {
		$err = text("err_unknown_type", $ds);
		$url .= "?cmd=zfs_destroy&zfs=$ds";
		my %param = ('error' => $err);
		webmin_log("Destroy", "dataset", "error: $err for $ds", \%param);
		goto OUT;
	}
	my $cmd = "zfs destroy '$ds'";
	@res = &backquote_command($cmd . " 2>&1");
	if ($? != 0) {
		# ошибка
		$err = join($sep, @res);
		$url .= "?cmd=$link&zfs=$ds";
		my %param = ('error' => $err);
		webmin_log("Destroy", "dataset", "error: $err for $ds", \%param);
		$err = text('err_destroy', $ds);
	} else {
		# ошибок нет, всё прошло
		webmin_log("Destroy", $obj, $cmd);
	}
} elsif (($in{'cmd'} eq 'atime') && has_access('uzfs_prop')) {
	# работаем
	my $ds = trim($in{'zfs'});
	my $atime = trim($in{'atime'});
	my $cmd;
	my @res = ();
	if ($atime ~~ ["on", "off"]) {
		$cmd = "zfs set atime=$atime '$ds'";
		$res = &backquote_command($cmd." 2>&1");
	} elsif ($atime eq "inherit") {
		$cmd = "zfs inherit atime '$ds'";
		$res = &backquote_command($cmd." 2>&1");
	} else {
		$err = "invalid atime value";
		$cmd = "zfs set atime=<error> '$ds'";
		$? = -1;
	}
	if ($? == 0) {
		# ошибок нет, всё прошло
		$url .= "?cmd=prop&zfs=$ds";
		webmin_log("Change", "atime", $cmd);
	} else {
		# ошибка
		if ($? != -1) {
			$err = join($sep, @res);
			my %param = ('error' => $err);
			webmin_log("Change", "atime", "error: $err for $ds", \%param);
			$err = text("err_prop_set", $in{'cmd'}, $atime, $ds);
		}
		$url .= "?cmd=atime&zfs=$ds";
	}
} elsif (($in{'cmd'} eq 'rdonly') && has_access('uzfs_prop')) {
	# работаем
	my $ds = trim($in{'zfs'});
	if (!zfs_allowed($ds)) {
		$url .= "cmd=prop&zfs=$ds";
		$err = text("err_forbidden", $ds),;
		my %param = ('error' => $err);
		webmin_log("Change", "readonly", "error: $err for ".$ds, \%param);
		goto OUT;
	}
	my $ro = trim($in{'readonly'});
	my $cmd;
	my @res = ();
	if ($ro ~~ ["on", "off"]) {
		$cmd = "zfs set readonly=$ro '$ds'";
		$res = &backquote_command($cmd." 2>&1");
	} elsif ($ro eq "inherit") {
		$cmd = "zfs inherit readonly '$ds'";
		$res = &backquote_command($cmd." 2>&1");
	} else {
		$err = "invalid readonly value";
		$cmd = "zfs set readonly=<error> '$ds'";
		$? = -1;
	}
	if ($? == 0) {
		# ошибок нет, всё прошло
		$url .= "?cmd=prop&zfs=$ds";
		webmin_log("Change", "readonly", $cmd);
	} else {
		# ошибка
		if ($? != -1) {
			$err = join($sep, @res);
			my %param = ('error' => $err);
			webmin_log("Change", "readonly", "error: $err for $ds", \%param);
			$err = text("err_prop_set", $in{'cmd'}, $ro, $ds);
		}
		$url .= "?cmd=rdonly&zfs=$ds";
	}
} elsif (($in{'cmd'} eq 'owner') && has_access('uzfs_prop')) {
	# работаем
	my $ds = trim($in{'zfs'});
	if (!zfs_allowed($ds)) {
		$url .= "cmd=prop&zfs=$ds";
		$err = text("err_forbidden", $ds),;
		my %param = ('error' => $err);
		webmin_log("Change", "owner", "error: $err for ".$ds, \%param);
		goto OUT;
	}
	my $user = trim($in{'user'});
	my $group = trim($in{'group'});
	if ($user eq "") {
		$err = "You must specify a username";
		my %param = ('error' => $err);
		my $owner = "<nouser>";
		if ($group ne "") {
			$owner .= ":".$group;
		}
		webmin_log("Change", "owner", "error: $err for $ds", \%param);
		$url .= "?cmd=owner&zfs=$ds";
		goto OUT;
	}
	$url .= "?cmd=";
	if (zfs_chown($user, $group, $ds) == 0) {
		$url .= "prop&zfs=$ds";
	} else {
		$url .= "owner&zfs=$ds";
	}
} elsif (($in{'cmd'} eq 'mode') && has_access('uzfs_prop')) {
	# работаем
	my $ds = trim($in{'zfs'});
	if (!zfs_allowed($ds)) {
		$url .= "cmd=prop&zfs=$ds";
		$err = text("err_forbidden", $ds),;
		my %param = ('error' => $err);
		webmin_log("Change", "mode", "error: $err for ".$ds, \%param);
		goto OUT;
	}
	my $md = oct(trim($in{'mode'}));
	if ($md > 0777) {
		$err = "The mode must be from 000 to 777 octal";
		my %param = ('error' => $err);
		webmin_log("Change", "mode", "error: $err for $ds", \%param);
		$url .= "?cmd=mode&zfs=$ds";
		goto OUT;
	}
	$url .= "?cmd=";
	if (zfs_chmod($md, $ds) == 0) {
		$url .= "prop&zfs=$ds";
	} else {
		$url .= "mode&zfs=$ds";
	}
} elsif (($in{'cmd'} eq 'compress') && has_access('uzfs_prop')) {
	# работаем
	my $ds = trim($in{'zfs'});
	if (!zfs_allowed($ds)) {
		$url .= "cmd=prop&zfs=$ds";
		$err = text("err_forbidden", $ds),;
		my %param = ('error' => $err);
		webmin_log("Change", "compression", "error: $err for ".$ds, \%param);
		goto OUT;
	}
	my $compress = trim($in{'compression'});
	my $cmd;
	my @res = ();
	if ($compress ~~ ["on", "off", "lzjb", "gzip", "gzip-1", "gzip-2",
		"gzip-3", "gzip-4", "gzip-5", "gzip-6", "gzip-7", "gzip-8",
		"gzip-9", "zle", "lz4"]) {
		$cmd = "zfs set compression=$compress '$ds'";
		$res = &backquote_command($cmd." 2>&1");
	} elsif ($compress eq "inherit") {
		$cmd = "zfs inherit compression '$ds'";
		$res = &backquote_command($cmd." 2>&1");
	} else {
		$err = "invalid compression value";
		$cmd = "zfs set compression=<error> $ds";
		$? = -1;
	}
	if ($? == 0) {
		# ошибок нет, всё прошло
		$url .= "?cmd=prop&zfs=$ds";
		webmin_log("Change", "compression", $cmd);
	} else {
		# ошибка
		if ($? != -1) {
			$err = join($sep, @res);
			my %param = ('error' => $err);
			webmin_log("Change", "compression", "error: $err for $ds", \%param);
			$err = text("err_prop_set", "compression", $compress, $ds);
		}
		$url .= "?cmd=compress&zfs=$ds";
	}
} elsif (($in{'cmd'} eq 'recsize') && has_access('uzfs_prop')) {
	# работаем
	my $ds = trim($in{'zfs'});
	if (!zfs_allowed($ds)) {
		$url .= "cmd=prop&zfs=$ds";
		$err = text("err_forbidden", $ds),;
		my %param = ('error' => $err);
		webmin_log("Change", "recordsize", "error: $err for ".$ds, \%param);
		goto OUT;
	}
	my $rsize = trim($in{'recordsize'});
	my $cmd = "zfs set recordsize=$rsize '$ds'";
	my @res = &backquote_command($cmd." 2>&1");
	if ($? == 0) {
		# ошибок нет, всё прошло
		$url .= "?cmd=prop&zfs=$ds";
		webmin_log("Change", "recordsize", $cmd);
	} else {
		# ошибка
		$err = join($sep, @res);
		$url .= "?cmd=recsize&zfs=$ds";
		my %param = ('error' => $err);
		webmin_log("Change", "recordsize", "error: $err for $ds", \%param);
			$err = text("err_prop_set", "recordsize", $rsize, $ds);
	}
} elsif (($in{'cmd'} eq 'volsize') && has_access('uvol_prop')) {
	# работаем
	my $ds = trim($in{'zfs'});
	if (!zfs_allowed($ds)) {
		$url .= "cmd=prop&zfs=$ds";
		$err = text("err_forbidden", $ds),;
		my %param = ('error' => $err);
		webmin_log("Change", "volsize", "error: $err for ".$ds, \%param);
		goto OUT;
	}
	my $vsize = trim($in{'volsize'});
	my $cmd = "zfs set volsize=$vsize refreservation=$vsize '$ds'";
	my @res = &backquote_command($cmd." 2>&1");
	if ($? == 0) {
		# ошибок нет, всё прошло
		$url .= "?cmd=prop&zfs=$ds";
		webmin_log("Change", "volsize", $cmd);
	} else {
		# ошибка
		$err = join($sep, @res);
		$url .= "?cmd=volsize&zfs=$ds";
		my %param = ('error' => $err);
		webmin_log("Change", "volsize", "error: $err for $ds", \%param);
		$err = text("err_prop_set", $in{'cmd'}, $vsize, $ds);
	}
} elsif (($in{'cmd'} eq 'quota') && has_access('uzfs_prop')) {
	# работаем
	my $ds = trim($in{'zfs'});
	if (!zfs_allowed($ds)) {
		$url .= "cmd=prop&zfs=$ds";
		$err = text("err_forbidden", $ds),;
		my %param = ('error' => $err);
		webmin_log("Change", "quota", "error: $err for ".$ds, \%param);
		goto OUT;
	}
	my $quota = trim($in{'quota'});
	if ($quota <= 0) {
		$quota = "none";
	}
	my $cmd = "zfs set quota='$quota' '$ds'";
	my @res = &backquote_command($cmd." 2>&1");
	if ($? == 0) {
		# ошибок нет, всё прошло
		$url .= "?cmd=prop&zfs=$ds";
		webmin_log("Change", "quota", $cmd);
	} else {
		# ошибка
		$err = join($sep, @res);
		$url .= "?cmd=quota&zfs=$ds";
		my %param = ('error' => $err);
		webmin_log("Change", "quota", "error $err for $ds", \%param);
		$err = text("err_prop_set", $in{'cmd'}, $quota, $ds);
	}
} elsif (($in{'cmd'} eq 'reservation') && has_access('uzfs_prop')) {
	# работаем
	my $ds = trim($in{'zfs'});
	if (!zfs_allowed($ds)) {
		$url .= "cmd=prop&zfs=$ds";
		$err = text("err_forbidden", $ds),;
		my %param = ('error' => $err);
		webmin_log("Change", "reservation", "error: $err for ".$ds, \%param);
		goto OUT;
	}
	my $reservation = trim($in{'reservation'});
	if ($reservation <= 0) {
		$reservation = "none";
	}
	my $cmd = "zfs set reservation='$reservation' '$ds'";
	my @res = &backquote_command($cmd." 2>&1");
	if ($? == 0) {
		# ошибок нет, всё прошло
		$url .= "?cmd=prop&zfs=$ds";
		webmin_log("Change", "reservation", $cmd);
	} else {
		# ошибка
		$err = join($sep, @res);
		$url .= "?cmd=reservation&zfs=$ds";
		my %param = ('error' => $err);
		webmin_log("Change", "reservation", "error $err for $ds", \%param);
		$err = text("err_prop_set", $in{'cmd'}, $reservation, $ds);
	}
} elsif  (($in{'cmd'} eq 'smb') && has_access('uzfs_prop')) {
	# работаем
	my $ds = trim($in{'zfs'});
	if (!zfs_allowed($ds)) {
		$url .= "cmd=prop&zfs=$ds";
		$err = text("err_forbidden", $ds),;
		my %param = ('error' => $err);
		webmin_log("Change", "sharesmb", "error: $err for ".$ds, \%param);
		goto OUT;
	}
	my $smb = trim($in{'sharesmb'});
	my $cmd = "zfs set sharesmb='$smb' '$ds'";
	my @res = &backquote_command($cmd." 2>&1");
	if ($? == 0) {
		# ошибок нет, всё прошло
		$url .= "?cmd=prop&zfs=$ds";
		webmin_log("Change", "sharesmb", $cmd);
	} else {
		# ошибка
		$err = join($sep, @res);
		$url .= "?cmd=smb&zfs=$ds";
		my %param = ('error' => $err);
		webmin_log("Change", "sharesmb", "error: $err for $ds", \%param);
		$err = text("err_prop_set", "sharesmb", $smb, $ds);
	}
} elsif (($in{'cmd'} eq 'nfs') && has_access('uzfs_prop')) {
	# работаем
	my $ds = trim($in{'zfs'});
	if (!zfs_allowed($ds)) {
		$url .= "cmd=prop&zfs=$ds";
		$err = text("err_forbidden", $ds),;
		my %param = ('error' => $err);
		webmin_log("Change", "sharenfs", "error: $err for ".$ds, \%param);
		goto OUT;
	}
	my $nfs = trim($in{'sharenfs'});
	my $cmd = "zfs set sharenfs='$nfs' '$ds'";
	my @res = &backquote_command($cmd." 2>&1");
	if ($? == 0) {
		# ошибок нет, всё прошло
		$url .= "?cmd=prop&zfs=$ds";
		webmin_log("Change", "sharenfs", $cmd);
	} else {
		# ошибка
		$err = join($sep, @res);
		$url .= "?cmd=nfs&zfs=$ds";
		my %param = ('error' => $err);
		webmin_log("Change", "sharenfs", "error: $err for $ds", \%param);
		$err = text("err_prop_set", "sharenfs", $nfs, $ds);
	}
} else {
	# команда не определена
	NOPERM:
	$err = "permission denied for $in{'cmd'} operation";
	my %param = ('error' => $err);
	webmin_log("Execute", $in{'cmd'}, "error: $err", \%param);
	$err = text('err_perm', $in{'cmd'});
	$url .= "?cmd=list";
}

OUT:
if ($err ne "") {
	$url .= "&err=".urlize($err);
}
redirect($url);
