#!/usr/bin/perl

use Text::Trim qw(trim);

require 'zfsmgr-lib.pl';
&ReadParse();
our (%pools, %dsctl);

# FIXME: убивает точку монтирования, чего делать не должен!
# максимальная строка dataset = 230 символов

sub zfs_dataset
{
	my ($ds, $prop) = @_;
	my $link = "<a href=\"?cmd=prop&zfs=$ds\">$ds</a>";
	my $mnt = $prop->{'mountpoint'};
	my $used = nice_size($prop->{'used'});
	my $avail = nice_size($prop->{'available'});
	my $quota = $prop->{'quota'};
	my $reservation = $prop->{'reservation'};
	my $compressratio = $prop->{'compressratio'};

# quota, recordsize, sharenfs, compression, readonly
# devices=off
# exec=off
# setuid=off
# dedup=off

# can edit properties?
# can create datasets?
#	push(@links, "<a href=\"create.pl?zfs=$ds\">Create</a>");
# can destroy dataset/pool?
#	push(@links, "<a href=\"destroy.pl?zfs=$ds\">destroy</a>");
	if ($quota <= 0) {
		$quota = $text{'quota_none'};
	} else {
		$quota = nice_size($quota);
	}
	if ($reservation <= 0) {
		$reservation = $text{'reservation_none'};
	} else {
		$reservation = nice_size($reservation);
	}

	print &ui_columns_row([$link, $mnt, $used, $avail, $quota,
		$reservation, $compressratio], undef);
}


sub zpool_show
{
	my ($pool, $health) = @_;
	my %list = zfs_filter($pool);

	print &ui_hidden_table_start("$pool ($health)", "width=100%", 5,
		$pool, 0);
	print &ui_columns_header([$text{'hdr_dataset'}, $text{'hdr_mountpoint'},
		$text{'hdr_used'}, $text{'hdr_avail'}, $text{'hdr_quota'},
		$text{'hdr_reservation'}, $text{'hdr_compressratio'}]);
	foreach $ds (sort(keys %list)) {
		zfs_dataset($ds, \%{$list{$ds}});
	}
	print &ui_hidden_table_end($pool);
}


sub zfs_error
{

	if (exists $in{'err'}) {
		print "<font color=\"red\">$in{'err'}</font>";
	}
}


sub zpool_error
{

	if (exists $in{'zerr'}) {
		print "<font color=\"red\">$in{'zerr'}</font>";
	}
}


sub zfs_footer
{
	my ($ds) = @_;

	zfs_error();
	print &ui_print_footer("", $text{"dataset_list"},
		"?zfs=$ds&cmd=prop", $text{"properties"});
}


sub zpool_footer
{
	my ($pool) = @_;

	zpool_error();
	print &ui_print_footer("?cmd=zlist", $text{"zpool_list"});
}


sub zfs_prop_link
{
	my ($url, $name, $propRef, $tagRef) = @_;
	my %prop = %{$propRef};
	my @tags = @{$tagRef};
	my $type = $prop{"type"}{"value"};
	my $val = $prop{$name}{"value"};
	my $src = $prop{$name}{"source"};
	my $name_str = $text{$name};
	my $src_str, $acc_name;

	if ($type eq "volume") {
		$acc_name = "uvol_prop";
	} else {
		$acc_name = "uzfs_prop";
	}

	if ($src eq "default") {
		$src_str = $text{"src_default"};
	} elsif ($src eq "local") {
		$src_str = $text{"src_local"};
	} elsif (index($src, "inherited from") == 0) {
		$src_str = &text("src_inherited", substr($src, 14));
	} else {
		if ($str eq "") {
			$str = "-";
		}
		$src_str = $str;
	}
	if ($val eq "") {
		$val = "-";
	}

	if (defined $url && has_access($acc_name)) {
		print &ui_columns_row(["<a href=\"?$url&val=$val\">$name_str</a>",
			$val, $src_str], \@tags);
	} else {
		print &ui_columns_row([$name_str, $val, $src_str], \@tags);
	}
}


sub zfs_owner_link
{
	my ($url, $ds, $propRef, $tagRef) = @_;
	my %prop = %{$propRef};
	my @tags = @{$tagRef};
	my $name = $text{"owner"};
	my $mnt = $prop{"mountpoint"}{"value"};
	my ($dev, $ino, $mode, $nlink, $uid, $gid, $rdev, $size, $atime,
		$mtime, $ctime, $blksize, $blocks) = lstat($mnt);
	my $owner = getpwuid($uid);
	my $grp = getgrgid($gid);
	my $val = "$owner:$grp";

	if (defined $url && has_access('uzfs_prop')) {
		print &ui_columns_row(["<a href=\"?$url&val=$val\">$name</a>",
			$val, undef], \@tags);
	} else {
		print &ui_columns_row(["$name", $val, undef], \@tags);
	}
}


sub zfs_mode_link
{
	my ($url, $ds, $propRef, $tagRef) = @_;
	my %prop = %{$propRef};
	my @tags = @{$tagRef};
	my $name = $text{"mode"};
	my $mnt = $prop{"mountpoint"}{"value"};
	my ($dev, $ino, $mode, $nlink, $uid, $gid, $rdev, $size, $atime,
		$mtime, $ctime, $blksize, $blocks) = lstat($mnt);
	my $val = sprintf "%o", $mode & 0777;

	if (defined $url && has_access('uzfs_prop')) {
		print &ui_columns_row(["<a href=\"?$url&val=$val\">$name</a>",
			$val, undef], \@tags);
	} else {
		print &ui_columns_row(["$name", $val, undef], \@tags);
	}
}


sub zfs_prop_show
{
	my ($ds) = @_;
	my $url = "zfs=$ds&cmd=";
	my %prop = zfs_get($ds, "all");
	my @tags = ("align=left width=30%",
		"align=left width=30% style=\"padding-left: 1em;\"",
		"align=left width=40%");
	my $ref = \%{$prop{$ds}};
	my $bstyle = "style=\"padding: 2px; background: blue; color: white; font-style: italic; text-align:";
	my $style2 = "$bstyle left; padding-left: 1em\"";
	my $style = "$bstyle left\"";
	my $is_pool = is_root($ds);
	my $type = $prop{$ds}{"type"}{"value"}, $type_str;
	my $is_vol = 0;

	if ($type eq "volume") {
		$type_str = $text{"type_volume"};
		$is_vol = 1;
	} elsif (($is_pool) && (index($ds, "/") < 0)) {
		$type_str = $text{"type_pool"};
	} else {
		$type_str = $text{"type_filesystem"};
	}
	print &ui_table_start("$ds ($type_str)", "width=100%", undef);
	print &ui_columns_header([$text{"property"}, $text{"value"}, $text{"source"}],
		[$style, $style2, $style]);

	my $mng = can_manage($ds);
	if (!$mng) {
		$is_pool = 1;
	}

	zfs_prop_link(undef, "mountpoint", $ref, \@tags);
	zfs_prop_link(undef, "recordsize", $ref, \@tags);
	zfs_prop_link(undef, "compression", $ref, \@tags);
#	if (!$is_vol) {
#	} else {
#		zfs_prop_link("${url}volsize", "volsize", $ref, \@tags);
#	}
#	if ($is_vol) {
#		zfs_prop_link(undef, "volblocksize", $ref, \@tags);
#	} else {
	zfs_prop_link($is_pool?undef:"${url}nfs", "sharenfs", $ref, \@tags);
	zfs_prop_link($is_pool?undef:"${url}quota", "quota", $ref, \@tags);
	zfs_prop_link($is_pool?undef:"${url}reservation", "reservation", $ref, \@tags);
	zfs_prop_link($is_pool?undef:"${url}rdonly", "readonly", $ref, \@tags);
	if (!$is_vol) {
		zfs_owner_link($is_pool?undef:"${url}owner", $ds, $ref, \@tags);
		zfs_mode_link($is_pool?undef:"${url}mode", $ds, $ref, \@tags);
	}
#	}
	print &ui_table_end();

	print &ui_table_start(undef, undef, 3);
	print "<tr>";
	if ($is_vol) {
		if (has_access('uvol_destroy')) {
			print "<td>";
			print &ui_form_start('index.cgi', 'post');
			print &ui_hidden('zfs', $ds);
			print &ui_hidden('cmd', 'zvol_destroy');
			print &ui_submit($text{"volume_destroy"});
			print &ui_form_end();
			print "</td>";
		}
	} else {
		if (has_access('uzfs_create') && $mng) {
			print "<td>";
			print &ui_form_start('index.cgi', 'post');
			print &ui_hidden('zfs', $ds);
			print &ui_hidden('cmd', 'zfs_create');
			print &ui_submit($text{"dataset_create"});
			print &ui_form_end();
			print "</td>";
		}
		if (has_access('uzfs_destroy') && !$is_pool && $mng &&
			($ref->{"usedbychildren"}{"value"} eq "0B")) {
			print "<td>";
			print &ui_form_start('index.cgi', 'post');
			print &ui_hidden('zfs', $ds);
			print &ui_hidden('cmd', 'zfs_destroy');
			print &ui_submit($text{"dataset_destroy"});
			print &ui_form_end();
			print "</td>";
		}
		if (has_access('uvol_create') && $mng) {
			print "<td>";
			print &ui_form_start('index.cgi', 'post');
			print &ui_hidden('zfs', $ds);
			print &ui_hidden('cmd', 'zvol_create');
			print &ui_submit($text{"volume_create"});
			print &ui_form_end();
			print "</td>";
		}
	}
	print "</tr>";
	print &ui_table_end();

	&ui_print_footer("", $text{"dataset_list"});
}


sub zfs_prop_sharenfs
{
	my ($ds, $val) = @_;

	print &ui_form_start('dataset.cgi', 'post');
	print &ui_hidden('zfs', $ds);
	print &ui_hidden('cmd', 'nfs');

	print &ui_table_start("$ds");

	print &ui_table_row($text{"sharenfs"}, &ui_textbox("sharenfs", $val, 48,
		undef, undef, "autofocus"));
	print &ui_table_row(undef, &ui_submit($text{"update"}));

	print &ui_table_end();
	print &ui_form_end();

	zfs_footer($ds);
}


sub ui_recordsize
{
	my ($val) = @_;

	print &ui_table_row($text{"recordsize"}, &ui_select("recordsize", $val,
		[["inherit", $text{"inherit"}], "8K", "128K", "1M"]));
}


sub ui_blocksize
{
	my ($val) = @_;

	print &ui_table_row($text{"recordsize"}, &ui_select("blocksize", $val,
		["512B", "1K", "2K", "4K", "8K", "16K", "32K", "64K", "128K"]));
}


sub ui_volsize
{
	my ($val) = @_;

	print &ui_table_row($text{"volsize"}, &ui_textbox("volsize", $val, 10));
}


sub ui_quota
{
	my ($val) = @_;

	if (!defined $val) {
		$val = "none";
	}
	print &ui_table_row($text{"quota"}, &ui_textbox("quota", $val, 8));
}


sub ui_reservation
{
	my ($val) = @_;

	if (!defined $val) {
		$val = "none";
	}
	print &ui_table_row($text{"reservation"}, &ui_textbox("reservation", $val, 8));
}


sub ui_reservation
{
	my ($val) = @_;

	if (!defined $val) {
		$val = "none";
	}
	print &ui_table_row($text{"reservation"}, &ui_textbox("reservation", $val, 8));
}


sub ui_sharenfs
{
	my ($val) = @_;

	print &ui_table_row($text{"sharenfs"}, &ui_textbox("sharenfs", $val, 48));
}


sub ui_compression
{
	my ($val) = @_;

	print &ui_table_row($text{"compression"}, &ui_select("compression", $val,
		[["inherit", $text{"inherit"}], "on", "off", "lz4", "lzjb", "gzip", "gzip-1", "gzip-2",
		"gzip-3", "gzip-4", "gzip-5", "gzip-6", "gzip-7", "gzip-8",
		"gzip-9", "zle"]));
}


sub ui_readonly
{
	my ($val) = @_;

	print &ui_table_row($text{"readonly"}, &ui_select("readonly", $val,
		[["inherit", $text{"inherit"}], "on", "off"]));
}


sub ui_owner
{
	my ($val) = @_;
	my @user = split(":", $val);

	print &ui_table_row($text{"user"}, &unix_user_input("user", $user[0]));
	print &ui_table_row($text{"group"}, &unix_group_input("group", $user[1]));
}


sub ui_mode
{
	my ($val) = @_;
#	my $md = $val;#sprintf "%o", $val & 0777;

	print &ui_table_row($text{"mode"}, &ui_textbox("mode", $val, 5, undef, 4));
}


sub zfs_prop_recordsize
{
	my ($ds, $val) = @_;

	print &ui_form_start('dataset.cgi', 'post');
	print &ui_hidden('zfs', $ds);
	print &ui_hidden('cmd', 'recsize');

	print &ui_table_start("$ds");

	ui_recordsize($val);
	print &ui_table_row(undef, &ui_submit($text{"update"}));

	print &ui_table_end();
	print &ui_form_end();

	zfs_footer($ds);
}


sub zfs_prop_volsize
{
	my ($ds, $val) = @_;

	print &ui_form_start('dataset.cgi', 'post');
	print &ui_hidden('zfs', $ds);
	print &ui_hidden('cmd', 'volsize');

	print &ui_table_start("$ds");

	ui_volsize($val);
	print &ui_table_row(undef, &ui_submit($text{"update"}));

	print &ui_table_end();
	print &ui_form_end();

	zfs_footer($ds);
}


sub zfs_prop_quota
{
	my ($ds, $val) = @_;

	print &ui_form_start('dataset.cgi', 'post');
	print &ui_hidden('zfs', $ds);
	print &ui_hidden('cmd', 'quota');

	print &ui_table_start("$ds");

	ui_quota($val);
	print &ui_table_row(undef, &ui_submit($text{"update"}));

	print &ui_table_end();
	print &ui_form_end();

	zfs_footer($ds);
}


sub zfs_prop_reservation
{
	my ($ds, $val) = @_;

	print &ui_form_start('dataset.cgi', 'post');
	print &ui_hidden('zfs', $ds);
	print &ui_hidden('cmd', 'reservation');

	print &ui_table_start("$ds");

	ui_reservation($val);
	print &ui_table_row(undef, &ui_submit($text{"update"}));

	print &ui_table_end();
	print &ui_form_end();

	zfs_footer($ds);
}


sub zfs_prop_compression
{
	my ($ds, $val) = @_;

	print &ui_form_start('dataset.cgi', 'post');
	print &ui_hidden('zfs', $ds);
	print &ui_hidden('cmd', 'compress');

	print &ui_table_start("$ds");

	ui_compression();
	print &ui_table_row(undef, &ui_submit($text{"update"}));
	print &ui_table_end();
	print &ui_form_end();

	zfs_footer($ds);
}


sub zfs_prop_atime
{
	my ($ds, $val) = @_;

	print &ui_form_start('dataset.cgi', 'post');
	print &ui_hidden('zfs', $ds);
	print &ui_hidden('cmd', 'atime');

	print &ui_table_start("$ds");

	print &ui_table_row("atime", &ui_select("atime", undef,
		["on", "off", ["inherit", $text{"inherit"}]]));
	print &ui_table_row(undef, &ui_submit($text{"update"}));

	print &ui_table_end();
	print &ui_form_end();

	zfs_footer($ds);
}


sub zfs_prop_readonly
{
	my ($ds, $val) = @_;

	print &ui_form_start('dataset.cgi', 'post');
	print &ui_hidden('zfs', $ds);
	print &ui_hidden('cmd', 'rdonly');

	print &ui_table_start("$ds");

	print &ui_table_row($text{"readonly"}, &ui_select("readonly", $val,
		[["inherit", $text{"inherit"}], "on", "off"]));
	print &ui_table_row(undef, &ui_submit($text{"update"}));

	print &ui_table_end();
	print &ui_form_end();

	zfs_footer($ds);
}


sub zfs_prop_owner
{
	my ($ds, $val) = @_;
	my $md = sprintf "%d", $val;

	print &ui_form_start('dataset.cgi', 'post');
	print &ui_hidden('zfs', $ds);
	print &ui_hidden('cmd', 'owner');

	print &ui_table_start("$ds");

	ui_owner($val);
	print &ui_table_row(undef, &ui_submit($text{"update"}));

	print &ui_table_end();
	print &ui_form_end();

	zfs_footer($ds);
}


sub zfs_prop_mode
{
	my ($ds, $val) = @_;
	my $md = sprintf "%d", $val;

	print &ui_form_start('dataset.cgi', 'post');
	print &ui_hidden('zfs', $ds);
	print &ui_hidden('cmd', 'mode');

	print &ui_table_start("$ds");

	ui_mode($val);
	print &ui_table_row(undef, &ui_submit($text{"update"}));

	print &ui_table_end();
	print &ui_form_end();

	zfs_footer($ds);
}


sub zfs_create
{
	my ($ds) = @_;

	print &ui_form_start('dataset.cgi', 'post');
	print &ui_hidden('zfs', $ds);
	print &ui_hidden('cmd', 'create');

	print &ui_table_start("$ds", undef, 2);
	print &ui_table_row($text{"dataset_new"},
		&ui_textbox("dataset", undef, 48, undef, undef, "autofocus"));
	ui_quota();
	ui_reservation();
	ui_recordsize("inherit");
	ui_sharenfs("off");
	ui_compression("inherit");
	ui_readonly("inherit");
	ui_owner("nobody:nobody");
	ui_mode(755);
	print &ui_table_row(undef, &ui_submit($text{"create"}));

	print &ui_table_end();
	print &ui_form_end();

	zfs_footer($ds);
}


sub zfs_destroy
{
	my ($ds) = @_;

	print &ui_form_start('dataset.cgi', 'post');
	print &ui_hidden('zfs', $ds);
	print &ui_hidden('cmd', 'destroy');

	print &ui_table_start("$ds", undef, 1);
	print &ui_table_row($text{"dataset_destroy_confirm"});
	print &ui_table_row("<font color=\"red\">"
		.$text{"destroy_last_chance"}."</font>");
	print &ui_table_row(undef, &ui_submit($text{"destroy"}));

	print &ui_table_end();
	print &ui_form_end();

	zfs_footer($ds);
}


sub zpool_create
{
	my ($ds) = @_;
	my $out = "";
	my %dsk = disk_info();
	my $i = 0;

	$out .= ui_form_start('zpool.cgi', 'post');
	$out .= ui_hidden('cmd', 'create');
	$out .= ui_table_start($text{'zpool_create'}, "width=\"100%\"", 1);
	$out .= ui_table_row($text{"zpool_new"},
		&ui_textbox("pool", $in{'pool'}, 32, undef, undef, "autofocus"));
	$out .= &ui_table_row($text{"zpool_type"},
		&ui_select("pool_type", $in{'pool_type'},
		[["stripe", "stripe"],
		["mirror", "mirror"],
		["raidz", "raidz"],
		["raidz2", "raidz2"],
		["raidz3", "raidz3"]]));
	$out .= &ui_columns_header([$text{'zpool_vdev'}, $text{'hdr_size'}, 
		$text{'hdr_vid'}, $text{'hdr_pid'}, $text{'hdr_serial'}]);

	foreach $vdev (sort(keys %dsk)) {
		my $ptr = \%{$dsk{$vdev}};
		my $disk = "disk_$i";
		$out .= &ui_columns_row([ui_checkbox($disk, $vdev, $vdev,
			defined($in{$disk})),
			nice_size($ptr->{'size'}), $ptr->{'vid'},
			$ptr->{'pid'}, $ptr->{'serial'}]);
		$i++;
	}
	$out .= ui_table_row(undef, ui_submit($text{"create"}, "create")."&nbsp;"
		.ui_submit($text{"import"}, "import")."&nbsp;"
		.ui_submit($text{"clear"}, "clear"), 8);

	$out .= &ui_table_end();
	$out .= ui_form_end();

	return $out;
}


sub zpool_destroy
{
	my ($pool) = @_;

	print &ui_form_start('zpool.cgi', 'post');
	print &ui_hidden('pool', $pool);
	print &ui_hidden('cmd', 'destroy');

	print &ui_table_start("$pool", undef, 1);
	print &ui_table_row($text{"zpool_destroy_confirm"});
	print &ui_table_row("<font color=\"red\">"
		.$text{"destroy_last_chance"}."</font>");
	print &ui_table_row(undef, &ui_submit($text{"destroy"}));

	print &ui_table_end();
	print &ui_form_end();
}


sub zvol_create
{
	my ($ds) = @_;

	print &ui_form_start('dataset.cgi', 'post');
	print &ui_hidden('zfs', $ds);
	print &ui_hidden('cmd', 'create');

	print &ui_table_start("$ds", undef, 2);
	print &ui_table_row($text{"volume_new"},
		&ui_textbox("volume", undef, 48, undef, undef, "autofocus"));
	ui_volsize();
	ui_blocksize("4K");
	ui_compression("inherit");
	ui_readonly("inherit");
	print &ui_table_row(undef, &ui_submit($text{"create"}));

	print &ui_table_end();
	print &ui_form_end();

	zfs_footer($ds);
}


sub zvol_destroy
{
	my ($ds) = @_;

	print &ui_form_start('dataset.cgi', 'post');
	print &ui_hidden('zfs', $ds);
	print &ui_hidden('cmd', 'destroy');

	print &ui_table_start("$ds", undef, 1);
	print &ui_table_row($text{"volume_destroy_confirm"});
	print &ui_table_row("<font color=\"red\">"
		.$text{"destroy_last_chance"}."</font>");
	print &ui_table_row(undef, &ui_submit($text{"destroy"}));

	print &ui_table_end();
	print &ui_form_end();

	zfs_footer($ds);
}


sub zfs_snapshot
{
	my ($ds) = @_;
	my $url = "zfs=$ds&cmd=";

	print &ui_table_start("$ds");

	print &ui_table_row("A snapshot name",
		&ui_textbox("snapshot", undef, 48, undef, undef, "autofocus"));
	print &ui_table_row(undef, &ui_button($text{"create"}));

	print &ui_table_end();
	zfs_footer($ds);
}


sub zfs_main
{

	if (defined($in{'zfs'}) && !zfs_allowed(trim($in{'zfs'}))) {
		$in{'err'} = text("err_forbidden", $in{'zfs'});
		goto DEFAULT;
	}

	if ($in{'cmd'} eq 'prop') {
	 	zfs_prop_show($in{'zfs'});
	} elsif ($in{'cmd'} eq 'nfs') {
		zfs_prop_sharenfs($in{'zfs'}, $in{'val'});
	} elsif ($in{'cmd'} eq 'smb') {
		zfs_prop_sharesmb($in{'zfs'}, $in{'val'});
	} elsif ($in{'cmd'} eq 'recsize') {
		zfs_prop_recordsize($in{'zfs'}, $in{'val'});
	} elsif ($in{'cmd'} eq 'quota') {
		zfs_prop_quota($in{'zfs'}, $in{'val'});
	} elsif ($in{'cmd'} eq 'reservation') {
		zfs_prop_reservation($in{'zfs'}, $in{'val'});
	} elsif ($in{'cmd'} eq 'compress') {
		zfs_prop_compression($in{'zfs'}, $in{'val'});
	} elsif ($in{'cmd'} eq 'atime') {
		zfs_prop_atime($in{'zfs'}, $in{'val'});
	} elsif ($in{'cmd'} eq 'rdonly') {
		zfs_prop_readonly($in{'zfs'}, $in{'val'});
	} elsif ($in{'cmd'} eq 'mode') {
		zfs_prop_mode($in{'zfs'}, $in{'val'});
	} elsif ($in{'cmd'} eq 'owner') {
		zfs_prop_owner($in{'zfs'}, $in{'val'});
	} elsif ($in{'cmd'} eq 'volsize') {
		zfs_prop_volsize($in{'zfs'}, $in{'val'});
	} elsif ($in{'cmd'} eq 'zfs_create') {
		zfs_create($in{'zfs'});
	} elsif ($in{'cmd'} eq 'zvol_create') {
		zvol_create($in{'zfs'});
	} elsif ($in{'cmd'} eq 'crt_snap') {
		zfs_snapshot($in{'zfs'});
	} elsif ($in{'cmd'} eq 'zfs_destroy') {
		zfs_destroy($in{'zfs'});
	} elsif ($in{'cmd'} eq 'zvol_destroy') {
		zvol_destroy($in{'zfs'});
	} else {
		goto DEFAULT;
	}
	return;

	DEFAULT:
	foreach $pool (sort(keys %pools)) {
		zpool_show($pool, $pools{$pool}{'health'});
	}
	zfs_error();
}


sub zpool_main
{
	my $out = "";
	my $footer = 0;

	if ($in{'cmd'} eq 'status') {
		# show pool status
		my $name = trim($in{'pool'});
	
		if (($name eq "") || !defined($pools{$name})) {
			goto DEFAULT;
		}
		my %pst = zpool_status($name);
		$out .= ui_table_start("zpool detail", undef, 1);
		$out .= ui_columns_row(["<pre>".$pst{'status'}."</pre>"]);
		my $frm = ui_form_start('index.cgi', 'post')
			.ui_hidden('pool', $name)
			.ui_hidden('cmd', 'zpool_destroy')
			.ui_submit($text{"zpool_destroy"})
			.ui_form_end();
		$out .= ui_columns_row([$frm], ["align=\"right\""]);
		$out .= ui_table_end();
		$footer = 1;
	} elsif ($in{'cmd'} eq 'zpool_destroy') {
		zpool_destroy($in{'pool'});
		$footer = 1;
	} elsif ($in{'cmd'} eq 'zpool_create') {
		$out .= zpool_create();
		$footer = 1;
	} else {
		DEFAULT:
		$out .= &ui_table_start(undef, "width=\"100%\"", 1);
		$out .= &ui_columns_header([$text{'hdr_pool'},
			$text{'hdr_size'}, $text{'hdr_alloc'},
			$text{'hdr_free'}, $text{'hdr_frag'}, $text{'hdr_cap'},
			$text{'hdr_dedup'}, $text{'hdr_health'}]);

		foreach $pool (sort(keys %pools)) {
			my $ptr = \%{$pools{$pool}};
			my $url = "?cmd=status&pool=$pool";
			$out .= &ui_columns_row([
				"<a href=\"$url\">".$pool."</a>",
				nice_size($ptr->{'size'}),
				nice_size($ptr->{'alloc'}),
				nice_size($ptr->{'free'}),
				$ptr->{'frag'}, $ptr->{'cap'}."%",
				$ptr->{'dedup'}, $ptr->{'health'}]);
		}
		my $frm = ui_form_start('index.cgi', 'post')
			.ui_hidden('cmd', 'zpool_create')
			.ui_submit($text{"zpool_create"})
			.ui_form_end();
		$out .= &ui_columns_row([$frm], ["colspan=\"7\""]);
		$out .= &ui_table_end();
	}

	print $out;
	if ($footer) {
		zpool_footer($ds);
	}
}

ui_print_header(undef, $text{'index_title'}, "", undef, 1, 1, 0,
	undef, undef, undef, $text{'index_version'} );

if (is_demo_mode()) {
	my @tabs = (['zfs', $text{'dataset_manager'}],
		['zpool', $text{'pool_manager'}]);
	my $selected = 'zpool';

	if (($in{'cmd'} eq "status") ||
		($in{'cmd'} eq "zpool_destroy") ||
		($in{'cmd'} eq "zpool_create") ||
		($in{'cmd'} eq "zlist")) {
		$selected = 'zpool';
	} else {
		$selected = 'zfs';
	}

	print ui_tabs_start(\@tabs, 'mode', $selected);

	print ui_tabs_start_tab('mode', 'zfs');
	zfs_main();
	print ui_tabs_end_tab('mode', 'zfs');

	print ui_tabs_start_tab('mode', 'zpool');
	zpool_main();
	print ui_tabs_end_tab('mode', 'zpool');
} else {
	zfs_main();
}

ui_print_footer("/", $text{'index'});
