BEGIN { push(@INC, ".."); };
use Data::Dumper;
use WebminCore;
use POSIX qw(strftime);
init_config();
foreign_require("mount", "mount-lib.pl");
my %access = &get_module_acl();
our %dsctl = zfs_list();
our %pools = zpool_list();

#print STDERR Dumper(\%pools);

sub is_demo_mode
{

	return has_access('demo');
}


sub has_access
{
	my ($acc) = @_;

	return $access{$acc};
}


sub zpool_list
{
	my %hash = ();
	my $cmd = "zpool list -Hpo name,size,alloc,free,frag,cap,dedup,health ";

	foreach my $ds (keys %dsctl) {
		my $pool = $dsctl{$ds}{"pool"};
		if (defined($hash{$pool})) {
			next;
		}
		my $line = &backquote_command($cmd.$pool);
		chomp $line;
		my ($name, $size, $alloc, $free, $frag, $cap, $dedup, $health) =
			split(" ", $line);
		$hash{$name}{"size"} = $size;
		$hash{$name}{"alloc"} = $alloc;
		$hash{$name}{"free"} = $free;
		$hash{$name}{"frag"} = $frag;
		$hash{$name}{"cap"} = $cap;
		$hash{$name}{"dedup"} = $dedup;
		$hash{$name}{"health"} = $health;
	}
	return %hash;
}


sub zpool_status
{
	my ($name) = @_;
#	my %hash = ();
	my $cmd = "zpool status -v $name";
	my @res = &backquote_command($cmd);
	my %hash = ();
	my $cfg = 0;

	chomp @res;
	$hash{'status'} = join("\n", @res);

	foreach $line (@res) {
		chomp ($line);
		if (($cfg eq 0) && ($line =~ /config:/)) {
			$cfg = 1;
		} elsif (($cfg eq 1) && ($line =~ /c.*t.*d.*/)) {
			my @l = split(" ", $line);
			$hash{'vdevs'}{$l[0]} = $l[1];
		}
	}

	return %hash;
}


# Returns max length of dataset name
sub zfs_max_length
{

	return 200;
}


sub zfs_list
{
	my ($zfs) = @_;
	my %hash=();
	my $cmd = "zfs list -Hpro name,mountpoint,type,used,available,compression,sharenfs,sync,quota,reservation,creation,usedbychildren,compressratio ";
	if (!defined($zfs) || ($zfs eq "")) {
		my @lst = split(",", $access{'mdataset'});
		my %plst = ();
		foreach my $ds (@lst) {
			my @zds = split("/", $ds);
			$plst{$zds[0]} = $zds[0];
		}
		$zfs = join(" ", keys %plst);
	}
	my @res = &backquote_command($cmd.$zfs);

	foreach my $line (@res) {
		chomp $line;
		my ($ds, $mnt, $type, $used, $avail, $compression, $sharenfs,
			$sync, $quota, $reservation, $creation, $usedbychildren,
			$compressratio)
			= split("\t", $line);
		my @dp = split("/", $ds);

		$hash{$ds}{"pool"} = $dp[0];
		$hash{$ds}{"mountpoint"} = $mnt;
		$hash{$ds}{"type"} = $type;
		$hash{$ds}{"used"} = $used;
		$hash{$ds}{"available"} = $avail;
		$hash{$ds}{"compression"} = $compression;
		$hash{$ds}{"sharenfs"} = $sharenfs;
		$hash{$ds}{"sync"} = $sync;
		$hash{$ds}{"quota"} = $quota;
		$hash{$ds}{"reservation"} = $reservation;
		$hash{$ds}{"creation"} = $creation;
		$hash{$ds}{"usedbychildren"} = $usedbychildren;
		$hash{$ds}{"compressratio"} = $compressratio;
	}

	return %hash;
}


sub zfs_filter
{
	my ($pool) = @_;
	my %list = ();

	foreach my $ds (keys %dsctl) {
		if ($dsctl{$ds}{"pool"} eq $pool) {
			$list{$ds} = $dsctl{$ds};
		}
	}

	return %list;
}


sub zfs_allowed
{
	my ($zfs) = @_;

	foreach my $ds (keys %dsctl) {
		if ($zfs =~ /^$ds$/i) {
			return 1;
		}
	}

	return 0;
}


# Returns 1 if specified dataset can be managed
# and 0 otherwise
sub can_manage
{
	my ($ds) = @_;
	my @lst = split(",", $access{'mdataset'});

	if (@lst == 0) {
		return 1;
	}
	foreach my $d (@lst) {
		if (($d eq $ds) || ($ds =~ /^$d\//i)) {
			return 1;
		}
	}

	return 0;
}


#zfs_get($dataset, $property)
sub zfs_get
{
	my ($zfs, $property) = @_;
	if (~$property) {my $property="all";}
	my %hash=();
	my $get=`zfs get -H $property '$zfs'`;

	open my $fh, "<", \$get;
	while (my $line = <$fh>) {
		chomp ($line);
		my($name, $property, $value, $source) = split(/\t/, $line);
		$hash{$name}{$property} = { value => $value, source => $source };
	}

	return %hash;
}


sub is_root
{
	my ($ds) = @_;
	my @res = split(",", $access{'mdataset'});

	if(grep(/^$ds$/i, @res)) {
		return 1;
	}

	return 0;
}


sub disk_info
{
	my %hash=();
	my $cmd = "/sbin/diskinfo -Hpc";
	my $list = &backquote_command($cmd);

	open my $fh, "<", \$list;
	while (my $line = <$fh>) {
		chomp ($line);
		my($type, $vdev, $vid, $pid, $serial, $size, $flags, $loc)
			= split(/;/, $line);
		$hash{$vdev} = { vid => $vid, pid => $pid, serial => $serial,
			size => $size, flags => $flags, location => $loc};
	}

	return %hash;
}

1;
