#!/usr/bin/perl

require 'zfsmgr-lib.pl';
&ReadParse();

use Text::Trim qw(trim);

my $url = "index.cgi";
my $zerr = "";
my $sep = "<br>";

sub disk_append
{
	my ($dref) = @_;
	my $ret = "";

	foreach my $disk (@{$dref}) {
		my $vdev = $in{$disk};
		$ret .= "&$disk=$vdev";
	}

	return $ret;
}


if (!is_demo_mode()) {
	goto NOPERM;
}

if ($in{'cmd'} eq "") {
	$zerr = $text{"err_empty_command"};
	my %param = ('error' => $zerr);
	webmin_log("Command", "undefined", trim($in{'zfs'}), \%param);
	$url .= "?cmd=list";
} elsif ($in{'cmd'} eq 'create') {
	my $type = trim($in{"pool_type"}), $name = trim($in{"pool"});
	my $str_type, $act = "Unknown";
	my @disks = grep{/disk_/} sort keys %in;

	$url .= "?";

	if (defined($in{'create'})) {
		if (($type ne "mirror") && ($type ne "raidz") &&
			($type ne "raidz2") && ($type ne "raidz3")) {
			$type = "";
			$str_type = "stripe";
		} else {
			$str_type = $type;
		}
		$act = "Create";
	} elsif (defined($in{'import'})) {
		$act = "Import";
	} elsif (defined($in{'clear'})) {
		$act = "Clear";
	} else {
		goto NOPERM;
	}

	if ((($act eq "Import") || ($act eq "Create")) && ($name eq "")) {
		$url .= "cmd=zpool_create";
		if ($str_type ne "") {
			$url .= "&pool_type=$str_type";
		}
		if ($act eq "Create") {
			$url .= disk_append(\@disks);
		}
		$zerr = $text{"err_empty_pool_name"};
		my %param = ('error' => $zerr);
		webmin_log($act, "pool", "error: $zerr ", \%param);
		goto OUT;
	}

	if (($act eq "Create") || ($act eq "Import")) {
		my @res = &backquote_command("zpool get -Hpo value name $name 2>&1");
		if ($? == 0) {
			$zerr = text("err_pool_exists", $name);
			$url .= "cmd=zpool_create&pool=$name";
			if ($act eq "Create") {
				$url .= disk_append(\@disks);
			}
			my %param = ('error' => $zerr);
			webmin_log($act, "pool", "error: $zerr for $name", \%param);
			goto OUT;
		}
	}

	my $zcmd = "zpool";
	if ($act eq "Create") {
		$zcmd .= " create $name $type";
		foreach my $disk (@disks) {
			my $vdev = $in{$disk};
			$zcmd .= " $vdev";
		}
	} elsif ($act eq "Import") {
		$zcmd .= " import $name";
	} elsif ($act eq "Clear") {
		my $vdevs;
		$zcmd .= " labelclear ";
		$url .= "cmd=zpool_create";
		foreach my $disk (@disks) {
			my $vdev = $in{$disk};
			@res = &backquote_command($zcmd."-f ".$vdev." 2>&1");
			if ($? != 0) {
				goto ERROR;
			}
			$vdevs .= " ".$vdev;
		}
		$xcmd .= $vdevs;
		goto OK;
	} else {
		goto OUT;
	}

	if (($act eq "Create") || ($act eq "Import")) {
		@res = &backquote_command($zcmd." 2>&1");
		if ($? != 0) {
			# error
			$url .= "cmd=zpool_create&pool=$name";
			if (($act eq "Import") &&
				($res[0] =~ /more than one matching pool/)) {
				@res = (text('err_import_demo', $name));
			} else {
				$url .="&pool_type=$str_type";
			}
			goto ERROR;
		}
	}
	$url .= "cmd=zlist";
	OK:
	# no errors, Ok
	webmin_log($act, "pool", $zcmd);
	goto OUT;

	ERROR:
	$zerr = join($sep, @res);
	$url .= disk_append(\@disks);
	my %param = ('error' => $zerr);
	webmin_log($act, "pool", "error: $zerr for $name", \%param);
	chomp @res;
	$zerr = join("<br>", @res);
} elsif ($in{'cmd'} eq 'destroy') {
	my $pool = trim($in{'pool'});
	if ($pool eq "") {
		$url .= "?cmd=zpool_destroy";
		$zerr = $text{"err_empty_pool_name"};
		my %param = ('error' => $zerr);
		webmin_log("Destroy", "pool", "error: $zerr ", \%param);
		goto OUT;
	}
	my @res = &backquote_command("zpool get -Hpo value name '$pool' 2>&1");
	if ($? != 0) {
		$zerr = join($sep, @res);
		$url .= "?cmd=zpool_destroy&pool=$pool";
		my %param = ('error' => $zerr);
		webmin_log("Get", "name", "error: $zerr for $pool", \%param);
		$zerr = text('err_pool_name', $pool);
		goto OUT;
	}
	chomp @res;
	my $cmd = "zpool destroy '$pool'";
	@res = &backquote_command($cmd . " 2>&1");
	if ($? != 0) {
		# ошибка
		$zerr = join($sep, @res);
		$url .= "?cmd=zpool_destroy&pool=$pool";
		my %param = ('error' => $zerr);
		webmin_log("Destroy", "zpool", "error: $zerr for $pool", \%param);
		$zerr = text('err_destroy', $pool);
	} else {
		# ошибок нет, всё прошло
		webmin_log("Destroy", "pool", $cmd);
		$url .= "?cmd=zlist";
	}
} else {
	# команда не определена
	NOPERM:
	$zerr = "permission denied for $in{'cmd'} operation";
	my %param = ('error' => $zerr);
	webmin_log("Execute", $in{'cmd'}, "error: $zerr", \%param);
	$zerr = text('err_perm', $in{'cmd'});
	$url .= "?cmd=zlist";
}

OUT:
if ($zerr ne "") {
	$url .= "&zerr=".urlize($zerr);
}
redirect($url);
